==> django-couchbase-0.0.1.zip: django-couchbase-0.0.1/django_couchbase/memcached.py <==
"""
Memcached cache backend for Django using pylibmc.

If you want to use the binary protocol, specify `'BINARY': True` in your CACHES
settings.  The default is `False`, using the text protocol.

pylibmc behaviors can be declared as a dict in `CACHES` backend `OPTIONS`
setting.

Unlike the default Django caching backends, this backend lets you pass 0 as a
timeout, which translates to an infinite timeout in memcached.
"""
import logging
import warnings
from threading import local

from django.conf import settings
from django.core.cache.backends.base import InvalidCacheBackendError
from django.core.cache.backends.memcached import BaseMemcachedCache

from couchbase import Couchbase,connection,exceptions

log = logging.getLogger('django.couchbase')

class CouchbaseCache(BaseMemcachedCache):
    def __init__(self, server, params, username=None, password=None):
        import os
        self._server = server
        self._client = None
        self._params = params
        super(CouchbaseCache, self).__init__(self._server, params, library=Couchbase,
                                           value_not_found_exception=ValueError)

    @property
    def _cache(self):
        # PylibMC uses cache options as the 'behaviors' attribute.
        # It also needs to use threadlocals, because some versions of
        # PylibMC don't play well with the GIL.
        client = self._client
        if client:
            return client
        host = None
        port = 8091
        if len(self._server) > 0:
            idx = self._server[0].find(':')
            host = self._server[0][:idx]
            port = self._server[0][idx+1:]

        if host == None:
            raise Exception('need host')

        client = self._lib.connect(  bucket= self._options.get('bucket', 'default'),
                                     host=host,
                                     password=self._options.get('password', ''),
                                     port=port,
                                     timeout=self._options.get('operation_timeout', 10 ),
                                     lockmode=connection.LOCKMODE_WAIT )

        self._client = client

        return client


    def get(self, key, default=None, version=None):
        try:
            rs = super(CouchbaseCache, self).get(key, default, version)
            return rs.value
        except Exception, e:
            #log.error('CouchbaseError: %s' % e, exc_info=True)
            return default



    def set(self, key, value, timeout=None, version=None):
        key = self.make_key(key, version=version)
        cn = self._cache
        rs = None
        
        try:
            cn.lock( key,  ttl=self._get_memcache_timeout(timeout) )
            
            rs = cn.set(key, value,
                         ttl=self._get_memcache_timeout(timeout)
                        )
        except exceptions.KeyExistsError,e:
            #pass
            #cn.replace( key, value, ttl=self._get_memcache_timeout(timeout) ) 
            log.error( str(e) )
        except exceptions.NotFoundError:
            #pass
            #log.error('ServerError saving %s (%d bytes)' % (key, len(value)),
            #          exc_info=True)
            
            rs = self._cache.add(key, value, ttl=self._get_memcache_timeout(timeout) )
        except Exception, e:
            log.error('MemcachedError: %s' % e, exc_info=True)
            rs = False

        return rs

    def add(self, key, value, timeout=0, version=None):
        key = self.make_key(key, version=version) 
        rs = self._cache.add(key, value, ttl=self._get_memcache_timeout(timeout) )
        return rs

    def close(self, **kwargs):
        pass

==> django-couchbase-0.0.2.zip: django-couchbase-0.0.2/django_couchbase/memcached.py <==
"""
Memcached cache backend for Django using pylibmc.

If you want to use the binary protocol, specify `'BINARY': True` in your CACHES
settings.  The default is `False`, using the text protocol.

pylibmc behaviors can be declared as a dict in `CACHES` backend `OPTIONS`
setting.

Unlike the default Django caching backends, this backend lets you pass 0 as a
timeout, which translates to an infinite timeout in memcached.
"""
import logging
import warnings
from threading import local

from django.conf import settings
from django.core.cache.backends.base import InvalidCacheBackendError
from django.core.cache.backends.memcached import BaseMemcachedCache

from couchbase import Couchbase,connection,exceptions

log = logging.getLogger('django.couchbase')

class CouchbaseCache(BaseMemcachedCache):
    def __init__(self, server, params, username=None, password=None):
        import os
        self._server = server
        self._client = None
        self._params = params
        super(CouchbaseCache, self).__init__(self._server, params, library=Couchbase,
                                           value_not_found_exception=ValueError)

    @property
    def _cache(self):
        # PylibMC uses cache options as the 'behaviors' attribute.
        # It also needs to use threadlocals, because some versions of
        # PylibMC don't play well with the GIL.
        client = self._client
        if client:
            return client
        host = None
        port = 8091
        if len(self._server) > 0:
            idx = self._server[0].find(':')
            host = self._server[0][:idx]
            port = self._server[0][idx+1:]

        if host == None:
            raise Exception('need host')

        client = self._lib.connect(  bucket= self._options.get('bucket', 'default'),
                                     host=host,
                                     password=self._options.get('password', ''),
                                     port=port,
                                     timeout=self._options.get('operation_timeout', 10 ),
                                     lockmode=connection.LOCKMODE_WAIT )

        self._client = client

        return client


    def get(self, key, default=None, version=None):
        try:
            rs = super(CouchbaseCache, self).get(key, default, version)
            return rs.value
        except Exception, e:
            #log.error('CouchbaseError: %s' % e, exc_info=True)
            return default



    def set(self, key, value, timeout=None, version=None):
        key = self.make_key(key, version=version)
        cn = self._cache
        rs = None
        
        try:
            #cn.lock( key,  ttl=self._get_memcache_timeout(timeout) )
            
            rs = cn.set(key, value,
                         ttl=self._get_memcache_timeout(timeout)
                        )
        except exceptions.KeyExistsError,e:
            #pass
            #cn.replace( key, value, ttl=self._get_memcache_timeout(timeout) ) 
            log.error( str(e) )
        except exceptions.NotFoundError:
            #pass
            #log.error('ServerError saving %s (%d bytes)' % (key, len(value)),
            #          exc_info=True)
            
            rs = self._cache.add(key, value, ttl=self._get_memcache_timeout(timeout) )
        except Exception, e:
            log.error('CouchbaseError: %s' % e, exc_info=True)
            rs = False

        return rs

    def add(self, key, value, timeout=0, version=None):
        key = self.make_key(key, version=version) 
        rs = False
        try:
            rs = self._cache.add(key, value, ttl=self._get_memcache_timeout(timeout) )
        except Exception, e:
            log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs

    def close(self, **kwargs):
        pass        

    def delete(self, key, version=None):
        key = self.make_key(key, version=version) 
        rs = False
        try:
            rs = self._cache.delete(key)
        except Exception, e:
            pass
            #log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs

    def incr(self, key, delta=1, version=None):
        key = self.make_key(key, version=version) 
        rs = False
        try:
            rs = self._cache.incr( key, delta )
        except exceptions.NotFoundError:
            rs = self._cache.add( key, delta, ttl=self._get_memcache_timeout(0) )
        except Exception, e:
            log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs        

    def decr(self, key, delta=1, version=None):
        key = self.make_key(key, version=version) 
        rs = False
        try:
            rs = self._cache.decr( key, delta )
        except exceptions.NotFoundError:
            rs = self._cache.add( key, delta, ttl=self._get_memcache_timeout(0) )
        except Exception, e:
            log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs

    def set_many(self, data, timeout=0, version=None):
        safe_data = {}
        for key, value in data.items():
            key = self.make_key(key, version=version)
            safe_data[key] = value
        rs = False    
        try:
            rs = self._cache.set_multi(safe_data, ttl=self._get_memcache_timeout(timeout))
        except Exception, e:
            log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs

    def delete_many(self, keys, version=None):
        l = lambda x: self.make_key(x, version=version)
        rs = False
        try:
            rs = self._cache.delete_multi( map(l, keys) )
        except Exception, e:
            log.error( 'CouchbaseError: %s' % e, exc_info=True )
        return rs
            

    def clear(self):
        '''
         couchbase-cli
           To clear all data in bucket. use command couchbase-cli on Couchbase Server, 
           /opt/couchbase/bin/couchbase-cli bucket-flush -u admin -p password -c 127.0.0.1:8091 -b bucket --force
          
         Couchbase Command Line tools 
           folder  /opt/couchbase/lib/python could be copied to other host
           or get from GitHub
             > git clone https://github.com/couchbase/couchbase-cli.git 
           execute:
             > python couchbase-cli bucket-flush -u admin -p password -c 192.168.12.13:8091 -b bucket --force

        '''
        log.error( 'CouchbaseError: please using couchbase-cli instead\n'
                   'couchbase-cli on GitHub\n'
                   'https://github.com/couchbase/couchbase-cli.git' )
        return False
